function [tem_diff,p,CRP] = SCEt_Behav_Ana
% SCEt_Behav_Ana
%
% loads behavioral data and generates temporal difference score,
% probability of recall, and CRPs across subjects. Performed for all
% subjects, use filter separately to exclude subjects.
%
% N. Long

  % If the behavioral data has been created and saved, load it. Otherwise run the subfunction below.

  if exist('~/Experiments/pyFR/pyFR_behav_scores.mat','file')
    load('~/Experiments/pyFR/pyFR_behav_scores.mat');
  else
    run_ecog_behav;
    load('~/Experiments/pyFR/pyFR_behav_scores.mat')
  end

% Extract prec, which is already calculated.
p = [exp.subj.precs]';

% Initialize the variables
CRP = nan(length(p),11);
tem_diff = nan(length(p),1);

for i = 1:length(p)
    
    % Extract lag crp for each subject, since doing it like prec will not
    % work
    lag_crps = [exp.subj(i).lag_crps];
    
    % Get the list length as this will correspond to the NAN of lag = 0 in
    % the CRP data.
    listLength = (length(lag_crps) + 1) / 2;
    
    % A few subjects are broken and don't have behavioral data, so
    % listLength > 5 skips those subjects
    if listLength > 5
        
        % Get the 5 lags before and after LAG = 0.
        CRP(i,:) = lag_crps(listLength-5:listLength+5);
        
        % Get the "high_tem" values, the CRP for lags +/- 1
        high_tem = nanmean(lag_crps(:,listLength-1:listLength+1),2);
        
        % Get the "low_tem" values, the CRP for lags +/- 3 through 5
        low_tem1 = nanmean(lag_crps(:,listLength-5:listLength-3),2);
        low_tem2 = nanmean(lag_crps(:,listLength+3:listLength+5),2);
        
        % Average the low tem values
        low_tem = nanmean([low_tem1,low_tem2],2);
        
        % Subtract low from high to get the difference score!
        tem_diff(i) = high_tem - low_tem;
    end    
end

% Montage-change subjects require that we load all of their behavioral data
% and average it together. See other documentation for explanation of
% montage subjects.
%

% TJ018... TJ018_1, TJ018_2
% TJ069... TJ069_1
% TJ078...TJ078_1

mont_subj = {'TJ018','TJ069','TJ078'};

for m = 1:length(mont_subj)
    [new_td,new_p,new_CRP] = fix_montage_changes(exp,mont_subj{m});
    
    ind = find(strcmp({exp.subj.id},mont_subj{m}));
    
    tem_diff(ind) = nanmean(new_td);
    p(ind) = nanmean(new_p);
    CRP(ind,:) = nanmean(new_CRP);
end

function [tem_diff,PREC_NEW,CRP] = fix_montage_changes(exp,subj)

mont(1,:) = find(strcmp({exp.subj.id},subj));
mont(2,:)  = find(strcmp({exp.subj.id},[subj '_1']));
try
    mont(3,:) = find(strcmp({exp.subj.id},[subj '_2']));
catch
   
end

PREC_NEW = nan(length(mont),1);
CRP = nan(length(mont),11);
tem_diff = nan(length(mont),1);

for i = 1:length(mont)
    
    lag_crps = [exp.subj(mont(i)).lag_crps];
    PREC_NEW(i) = [exp.subj(mont(i)).precs];
    listLength = (length(lag_crps) + 1) / 2;
    
    if listLength > 5
        CRP(i,:) = lag_crps(listLength-5:listLength+5);
        high_tem = nanmean(lag_crps(:,listLength-1:listLength+1),2);
        low_tem1 = nanmean(lag_crps(:,listLength-5:listLength-3),2);
        low_tem2 = nanmean(lag_crps(:,listLength+3:listLength+5),2);
        low_tem = nanmean([low_tem1,low_tem2],2);
        tem_diff(i) = high_tem - low_tem;
    end
end

function run_ecog_behav
	% Finds SPC, CRP, precs, tem/sem factor and PFR for ecog data. Does it
% subject by subject because there is no mega data struct for ecog
% patients. Does not make plots since it's individual subjects. Use
% plot_ecog_behav for plots.
%
% N. Long

% Define experiment
params.exp = 'pyFR';
params.test = 0;

% Get subjects for experiment
subj = get_subs(params.exp);
% Put subjects in a format for apply_to_subj
for s = 1:length(subj)
    exp.subj(s).id = subj{s};
end

% Submit each subject as their own job.

if params.test
    exp.subj = apply_to_subj(exp.subj, @behav_pyFR, {params});
else
    parpool(12)
    parfor i=1:length(exp.subj)
        try
            subj = behav_pyFR(exp.subj(i),params);
        catch e
            fprintf('\n*****\nERROR!!!! : %s\n******\n',e.message);
        end
    end
    
    delete(gcp('nocreate'))
end

% Save out exp
savedir = '~/Analyses/behavioral/';
savefile = ([params.exp '_behav_scores']);
save(fullfile(savedir,savefile),'exp');

function subj = behav_pyFR(subj,params)
% Initialize structure
subj.precs = nan;
subj.temp_facts = nan;
subj.SPC = nan;
subj.lag_crps = nan;

% Get subject events
[events,~] = get_sub_events(params.exp,subj.id);

try
    % Convert events to data structure for use with behavioral toolbox
    % codes
    data = FRdata(events,'list');

    subj.precs = p_rec(data.recalls,data.subject,data.listLength);
    subj.temp_facts = temp_fact(data.recalls,data.subject,data.listLength);
    subj.SPC = spc(data.recalls,data.subject,data.listLength);
    subj.lag_crps = crp(data.recalls,data.subject,data.listLength);
catch
    fprintf('%s\n','not enough recall events');
end
