function params = SCEt_Default_Params(params)
% SCEt_Default_Params
%
% Sets all parameters necessary for spectral analysis. Analogous to a
% config file.
%
% INPUTS params    a structure with desired analysis parameters, if you
%           want to overwrite the defaults. If not, submit an empty structure
%
% OUTPUTS params    a structure with analysis parameters. Anything inputted
%           will override defaults, any non-specific parameters will use the
%           default values.
% N. Long

% Set the analysis an data directory. These vary as a function of the
% machine you are on
anadir = '~/Analyses/';
datadir = '/Volumes/nicole_ssd_RAID/data/';

% Set the defaults. These will be used if params is empty.
defaults = struct();
defaults.expdir = '~/Experiments/pyFR/';    % Where the experiment is located, contains things like WAS matrix

% Which analysis to run - if none is specified, by default grabs encoding
% data and does the SME analysis
if ~isfield(params,'analyses')
    fprintf('%s\n','WARNING: no analysis defined. Doing basic SME');
    params.encoding = 1;
    defaults.analyses = SCEt_Ana_Conds('rec');
end
% No default channels for pyFR since each subject is different
defaults.chans = [];
% Default buffer
defaults.buffms = 1000;

% Defaults specific to the "phase" (encoding/retrieval) of analysis
if params.encoding == 1
    defaults.durms = 1600;
    defaults.offset = 0;
    defaults.tbinsMS = {[defaults.offset defaults.offset+defaults.durms]};
    defaults.studydir = ([anadir 'encoding/ecog/']);
    defaults.powdir = ([datadir 'pyFR_encoding/']);
    defaults.figdir = ([defaults.expdir 'encoding/figs']);
else
    defaults.durms = 4000;
    defaults.offset = -2000;
    defaults.tbinsMS = {[defaults.offset defaults.offset+defaults.durms]};
    defaults.studydir = ([anadir 'retrieval/ecog/']);
    defaults.powdir = ([datadir 'pyFR_retrieval/']);
    defaults.figdir = ([defaults.expdir 'retrieval/figs']);
end

defaults.resampledrate = 200;               % Resamples data BEFORE extracting power
defaults.freqs = eeganalparams('freqs');    % Freqs to ana - requires eeganalparams.txt file in ~/eeg/
defaults.width = eeganalparams('width');    % Width of wavelet - "
defaults.iterations = 1000;                 % Iterations for shuffling procedure
defaults.fbins = {[3 4],[6 8.0001],[3 8.0001],[10 14.0001],[16 26.0001],[28 42.0001],[70 160]};
defaults.fbinslabel = {'Theta1','Theta2','Theta','Alpha','Beta','Gamma1','Gamma2'};

defaults.pval = .05;        % Threshold for significance testing
defaults.window = 100;      % Average over WINDOW milliseconds, for moving average
defaults.slide = 50;        % Slide window every x milliseconds, for moving average
defaults.logtransform = 1;  % Perform log transform

% Merge default and inputted params
params = propval(params,defaults,'strict',false);

% Location of the WAS matrix. Is experiment specific
params.wasloc = ([params.expdir 'FR_WAS_matrix.mat']);

% Get all the time bins
msbins = [];
for s = params.offset:params.slide:(params.durms-(abs(params.offset))-params.window)
    msbins = [msbins;s,s+params.window];
end
% Get the labels, based MS Get the lower and upper bounds of the msbins
for t = 1:length(params.tbinsMS)
    
    lower = params.tbinsMS{t}(1);
    upper = params.tbinsMS{t}(2);
    
    label = strcat([mat2str(lower) 'to' mat2str(upper) 'ms']);
    params.tbinslabel(t) = {label};
    
    LInd = find(msbins(:,1) == lower);
    UInd = find(msbins(:,2) == upper);
    
    % Get all the non-overlapping sample bins
    sbins = [];
    for i = LInd:2:UInd
        sbins = [sbins;i];
    end
    
    params.tbins{t} = sbins';
end

% Build the tbin labels
for t = 1:length(params.tbinsMS)
    label = strcat([mat2str(params.tbinsMS{t}(1)) 'to' mat2str(params.tbinsMS{t}(2))]);
    params.tbinslabel(t) = {label};
end

params.time = ((params.durms/params.window)*(params.window/params.slide))-((params.window/params.slide)-1);