function [SCE,HT,LT,NR,R,SME,HG] = SCEt_Figure2(roi,params)
% Generates Figure 2 from Long & Kahana, Sucessful memory formation
% is driven by contextual encoding in the core memory network
% Requires that SCEt_Main_Analysis with params.tstat = 0 have been run
%
% INPUTS
% roi           ROI to analyze, format, roi = 'Hipp'; corresponds to ROIs
%               in bipolar_ROI
% params        structure with analysis parameters. run SCEt_Default_Params
%               to generate some parameters
%
% OUTPUTS
%
% SCE           Difference in zpower between sub. clustered and
%               non-clustered (but recalled) items, for the "freq_of_int" and
%               "time_of_int"
% HT            Zpower freq/time of int for temporally clustered items
% LT            " not temporally clustered
% NR            " not recalled
% R             " recalled
% SME           Same as SCE, but for R > NR
% HG            Average high gamma across all trials
%
% Will generate a bar plot of the SCE across 6 freqs and the time_of_int
%
% N. Long

% Set the filename. This function always relies on tstat, so we can hard
% code the "stat/" and "_stat/" into the filename.
if params.tstat
    filename = ([params.studydir 'stat/' params.analyses(1).name '_stat/Stat_bipolar_' roi '_']);
    TIME = length(params.tbins);
    FREQ = length(params.fbins);
else
    filename = ([params.studydir 'TF/' params.analyses(1).name '/Stat_bipolar_' roi '_']);
    TIME = params.time;
    FREQ = length(params.freqs);
end

% Load the subjects. Filter to the good subjects and those with >=5 events
% per cond
subj = get_subs('pyFR');
load ~/Experiments/pyFR/indices.mat
fullInd = montInd & minInd& techInd & leftInd;
subj = subj(fullInd);

% Initialize the variables
HT = nan(length(subj),TIME,FREQ);
LT = nan(length(subj),TIME,FREQ);
NR = nan(length(subj),TIME,FREQ);
R = nan(length(subj),TIME,FREQ);
if params.tstat
    HG = nan(length(subj),TIME);
else
HG = nan(length(subj),((TIME+1)/2)); % HG already reduces the extra dimensions in the original analysis.
end

% Get the data across subjects, but only if the files exist.
for s = 1:length(subj)
    if exist ([filename subj{s} '.mat'],'file');
        data = load ([filename subj{s} '.mat']);
        HT(s,:,:) = data.C1;
        LT(s,:,:) = data.C2;
        R(s,:,:) = data.C12;
        NR(s,:,:) = data.C3;
        HG(s,:) = squeeze(nanmean(data.HIGHGAMMA));
    end
end

% Get the SCE/SME by subtracting HT and LT or R and NR, and only extracting
% for the time of interest
SME = squeeze(R(:,params.time_of_int,:) - NR(:,params.time_of_int,:));
SCE = squeeze(HT(:,params.time_of_int,:) - LT(:,params.time_of_int,:));

% Make the figures
close all
% Do some t-tests to get stars for significance
[~,pval,~,~] = ttest(SCE);

% Set parameters to use with barweb
params.sig = pval < .05;
params.error_width = 3;
params.xlim = [0 length(params.fbins)+1];
params.ylim = [-.2 .2];
params.xticklabel = {'TL','TH','A','B','GL','GH'};
params.ylabel=  'Z-scored power difference';
params.ytick = [params.ylim(1) 0 params.ylim(2)];

% Get means and SEM because LM has variance issues.
D = nanmean(SCE);
d = SCE;
% EB1 = LoftusMasson(d1);
df = length(find(~isnan(SCE(:,1))==1));
EB = nanstd(d)/sqrt(df);

% Don't necessarily plot since we use this function to extract data for
% SCEt_Figure4.
if params.plot
% Plot it!
barweb(D,EB,params);
% Print it if that's what we want
if params.print
    figname = (['ecog_encoding_' params.tbinslabel{params.time_of_int}  '_'...
        ana '_' roi '.eps']);
    print(fullfile(params.figdir,figname),'-depsc2');
    
    % Convert EPS to PDF
    convert_eps_to_pdf(params.figdir,pwd);
else
    pause
end
end

% Squeeze everything to freq/time of interest for the outputs
SCE = squeeze(SCE(:,params.freq_of_int));
SME = squeeze(SME(:,params.freq_of_int));
HT = squeeze(HT(:,params.time_of_int,params.freq_of_int));
LT = squeeze(LT(:,params.time_of_int,params.freq_of_int));
R = squeeze(R(:,params.time_of_int,params.freq_of_int));
NR = squeeze(NR(:,params.time_of_int,params.freq_of_int));
HG = squeeze(HG(:,params.time_of_int));

