function [time_of_hfa_diff] = SCEt_Figure3(roi,params)
% Generates Figure 3 from Long & Kahana, Sucessful memory formation
% is driven by contextual encoding in the core memory network
% Requires that SCEt_Main_Analysis with params.tstat = 0 have been run
%
% INPUTS
% roi           roi to analyze, e.g. roi = 'Hipp'
% params        structure of parameters
%
% OUTPUTS
% time_of_hfa_diff      peak time for each subject. returns a single number
%                       for each subject corresponding to the time interval at which the SCE is
%                       maximal. 
% N. Long

close all

% Do you want a box-and-whisker of interquartile range below the time
% course?
do_box_plot = 1;

% Get the subjects and appropriate filters
subj = get_subs('pyFR');
load ~/Experiments/pyFR/indices.mat
fullInd = montInd & minInd & techInd & leftInd;
subj = subj(fullInd);

% Define directories/variables
% Note: This code is intended to be run with params.tstat = 0

if params.tstat
    filename = ([params.studydir 'stat/' params.analyses(1).name '_stat/Stat_bipolar_' roi '_']);
    TIME = length(params.tbins);
    FREQ = length(params.fbins);
    timebins = TIME;
else
    filename = ([params.studydir 'TF/' params.analyses(1).name '/Stat_bipolar_' roi '_']);
    TIME = params.time;
    FREQ = length(params.freqs);
    timebins = (TIME+1)/2; % If we're doing all the time intervals, 
    % every other interval is an overlap, so we want to remove those 
end

% Define the frequency (gamma) to be used. 
f = find(strcmp(params.fbinslabel,'Gamma2')) ;
freq_of_int = params.freqs >= params.fbins{f}(1) & params.freqs <= params.fbins{f}(2);

% Initialize variables.
C1 = nan(length(subj),TIME,FREQ);
C2 = nan(length(subj),TIME,FREQ);
C3 = nan(length(subj),TIME,FREQ);
time_of_hfa_diff= nan(length(subj),1);

for s = 1:length(subj)
    if exist([filename subj{s} '.mat'],'file');
        data = load([filename subj{s}]); 
        C1(s,:,:) = squeeze(data.C1);
        C2(s,:,:) = squeeze(data.C2);
        C3(s,:,:) = squeeze(data.C3);        
    end
end

% Get just the HFA signal across time for all conds. Note we use 1:2:TIME
% to get the non-overlapping intervals. 
HFA_C1 = squeeze(nanmean(C1(:,1:2:TIME,freq_of_int),3));
HFA_C2 = squeeze(nanmean(C2(:,1:2:TIME,freq_of_int),3));
HFA_C3 = squeeze(nanmean(C3(:,1:2:TIME,freq_of_int),3));

% Get the number of S in each condition for the SEM EBs
n_hfa_C1=length(find(~isnan(HFA_C1(:,1))));
n_hfa_C2=length(find(~isnan(HFA_C2(:,1))));
n_hfa_C3=length(find(~isnan(HFA_C3(:,1))));

fprintf('Num sub %s',mat2str(n_hfa_C1));fprintf('\n');

% Get the SCE
HFA_D = HFA_C1 - HFA_C2;

% For all the subjects, loop through the difference values and find the
% time interval at which HFA_D is maximal.
for ii=1:length(subj)
    DIFF = HFA_D(ii,:);
    try
        time_of_hfa_diff(ii) = find(max(DIFF)==DIFF);
    end
end

% SEM EBs
eb_HFA_C1 = nanstd(HFA_C1)./sqrt(n_hfa_C1);
eb_HFA_C2 = nanstd(HFA_C2)./sqrt(n_hfa_C2);
eb_HFA_C3 = nanstd(HFA_C3)./sqrt(n_hfa_C3);

% Colors of the lines. 
colors{1} = [.15 .15 .15]; % Black
colors{2} = [.35 .35 .35]; % Dark Gray
colors{3} = [.8 .8 .8];    % Light Gray

% colors{1} = [1 .5 0]; % Black
% colors{2} = [.35 .35 .35]; % Dark Gray
% colors{3} = [.3 .5 .7];    % Light Gray

if do_box_plot
    % Makes a subplot below the time course of the interquartile range.
    subplot(2,1,2,'position',[.1 0 .7 .2]);
    boxplot(time_of_hfa_diff,'orientation','horizontal','colors',[0 0 0]);
    box off
    set(gca,'xlim',[0 20]);
   % set(gca,'xtick',[2.5 8 13 19]);
    set(gca,'xticklabel',[]);
    set(gca,'ytick',[]);
    subplot(2,1,1,'position',[.1 .3 .7 .6]);    
end

% Get the median (Q2) and mode (Q3)
Q2 = nanmedian(time_of_hfa_diff);
Q3 = mode(time_of_hfa_diff);

% Plot the error bars, use plotSEM instead of errorbar because it does
% shading
plotSEM(1:timebins,nanmean(HFA_C1),eb_HFA_C1,colors{1}); hold on;
plotSEM(1:timebins,nanmean(HFA_C2),eb_HFA_C2,colors{2});
plotSEM(1:timebins,nanmean(HFA_C3),eb_HFA_C3,colors{3});

% Plot the actual lines. All black/dashed
plot(nanmean(HFA_C1),'color',[0 0 0],'Linewidth',2,'Linestyle','--');
plot(nanmean(HFA_C2),'color',[0 0 0],'Linewidth',2,'Linestyle','--');
plot(nanmean(HFA_C3),'color',[0 0 0],'Linewidth',2,'Linestyle','--');

% Add circles for the median and mode
plot(Q2,.45,'marker','o','markersize',10,'color','black');
plot(Q3,.45,'marker','.','markersize',35,'color','black');

set(gca,'Fontsize',params.fontsize);
box off

% Miscellaneous plot parameters.
ylim_min = -.5;ylim_max = .5;
set(gca,'ylim',[ylim_min ylim_max]);
set(gca,'xlim',[1 timebins]);
set(gca,'ytick',[ylim_min 0 ylim_max]);
set(gca,'xtick',[1 3 8 13 18]);
set(gca,'xticklabel',{'-300' '0' '500' '1000' '1500'});

% Make a horizontal line at y = 0.
line([0 timebins],[0 0],'color','black');
% Make a vertical line at x = 3, where the word onset occurs (= 0 ms)
line([3 3],[ylim_min ylim_max],'color','black');

% Print if desired.
if params.print
    figname = (['pyFR_HT_LT_zpow_' roi '_time_highgamma.eps']);
    print(fullfile(params.figdir,figname),'-depsc2');
    convert_eps_to_pdf(params.figdir,pwd);
else
    keyboard%pause
end

