function SCEt_Main_Analysis
% This code runs the analyses in Long & Kahana, Sucessful memory formation
% is driven by contextual encoding in the core memory network
%
% A few notes:
%   Can run "tstat" analysis: finds the difference in z-scored power
%   between two conditions for specific time and frequency bins (e.g. 0-500
%   ms / 4 - 8Hz) OR a "TF" analysis: same differences for "all" time and
%   frequencies (e.g. 100 ms bins from -300 to 1600 ms / 2 - 100 Hz)
%   Only analyzes iEEG, not scalp EEG, data.
% 	Only analyzes bipolar re-referenced data.
%
% Requires eeg_toolbox, eeg_ana, iEEGDataBase
%
% N. Long 4.9.15

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
params = struct();
params.exp = 'pyFR';            % Define your experiment

files = {'HT_LT_NR'};           % Define the contrast. MUST exist in stat_conditions_EXPNAME
params.encoding = 1;            % Encoding (1) or recall (0)
params.tbinsMS = {[-300 0],...
    [0 500],[500 1000],...
    [1000 1500],[0 1600]};      % Time bins to use if NOT doing time-frequency

params.roi = {'LIFG','LIT','Hipp'};

params.durms = 1900;           % Duration (in MS) around an event you want to analyze
params.offset = -300;          % How much pre-stimulus time to analyze
params.resampledrate = 200;    % Resample rate for power analysis

params.tstat = 0;               % Only get certain time bins/freq bands (1) or do whole time-freq spectrogram (0)
params.shuffle = 0;             % Run permutation test (1) or not (0)
params.N = 136;                 % N subjects to run

params.zscoretype = 'zscore_session';
params.electype = 'bipolar_power';

params.min_ev = 0;              % Minimum number of events required for analysis
params.overwrite = 0;           % Overwite files (1) or don't (0) if they already exist
params.test = 0;                % Run in serial for testing (1) or in parallel (2)

% Per's frequency definitions, except theta changed.
params.fbins = {[3 4],[6 8.0001],[10 14.0001],[16 26.0001],[28 42.0001],[44 100]};
params.fbinslabel = {'Theta1','Theta2','Alpha','Beta','Gamma1','Gamma2'};

% Which analyses do you want to run?
params.run_pow = 1;
params.run_roi = 1;

% Which figures do you want to generate?
params.gen_fig1 = 0; % Behavioral
% Below are neural and require run_pow and run_roi to be done.
params.gen_fig2 = 0;
params.gen_fig3 = 0;
params.gen_fig4 = 0;

% Parameters consistent across all figures
params.freq_of_int = 6; % Which frequency band to analyze?
params.fontsize = 30;

%close any pools that are open
delete(gcp('nocreate'))

% Get the subject structure &  Put it in a format that apply_to_subj understands
subj = get_subs(params.exp);

for s = 1:length(subj)
    exp.subj(s).id = subj{s};
end

% Define the appropriate condition and parameter files. These define the
% conditions to analyze and the experiment-specific parameters,
% respectively.
%
% stat_conditions are located in  ~/nl_scripts/trunk/*/CondFilts
% default_params are location in ~/nl_scripts/trunk/*/Defaults

% make_conds = strcat(['stat_conditions_' params.exp]);
% cond_handle = str2func(make_conds);
%
% default_params = strcat(['default_local_' params.exp '_params']);
% default_handle = str2func(default_params);

% Get the conditions to run. There is undoubtedly a better way to do this
for f = 1:length(files)
    a = SCEt_Ana_Conds(files{f});
    params.analyses(f) = a;
end

% Load default parameters. This sets up which analyses to run (e.g. SME)
% and sets the various directories. Similar to Burke's config files.
params = SCEt_Default_Params(params);

if params.test && params.run_pow == 1
    exp.subj = apply_to_subj(exp.subj, @run_pow, {params});
elseif params.run_pow == 1
    parpool(12)
    parfor i=1:length(exp.subj)
        try
            run_pow(exp.subj(i),params);
        catch e
            fprintf('\n*****\nERROR!!!! : %s\n******\n',e.message);
        end
    end
    delete(gcp('nocreate'))
end

if params.test && params.run_roi == 1
    exp.subj = apply_to_subj(exp.subj, @run_roi, {params});
elseif params.run_roi == 1
    parpool(12)
    parfor i=1:length(exp.subj)
        try
            run_roi(exp.subj(i),params);
        catch e
            fprintf('\n*****\nERROR!!!! : %s\n******\n',e.message);
        end
    end
    delete(gcp('nocreate'))
end

if params.gen_fig1
    params.print = 0;       % Do you want to print the figures?
    SCEt_Figure1(params);
end

if params.gen_fig2
    % Specify a few parameters here:
    params.print = 0;
    params.tstat = 1;       % Should be run with tstat = 1 because it is not designed to do TF plots.
    params.time_of_int = 5; % Which time interval to analyze?
    params.plot = 1;        % Plot = 1 b/c we want the figure, not just the data.
    for r = 1:length(params.roi)
        SCEt_Figure2(params.roi{r},params);
    end
end

if params.gen_fig3
    % Specify a few parameters here:
    params.print = 0;               % Do you want to print the figures?
    params.tstat = 0;               % Need all the time intervals if plotting time!
    params.roi_of_int = 'Hipp';     % Which ROI?    
    for r = 1:length(params.roi)
        SCEt_Figure3(params.roi{r},params);
    end
end

if params.gen_fig4
    % Specify a few parameters here:
    params.print = 0;               % Do you want to print the figures?
    params.tstat = 0;               % Need full range of time for HFA
    params.time_of_int = 17;        % Which time interval to analyze?
    params.roi_of_int = 'Hipp';     % Which ROI?
    params.plot = 0;
    SCEt_Figure4(params);
end

function subj = run_pow(subj,params)
% Step one of a time frequency analysis on bipolar electrodes comparing two
% conditions using t-tests.
% INPUTS
% subj      structure of the form: subj.id = 'BW001'
% params    structure with analysis parameters

% Get events and channels
[events] = get_sub_events(params.exp,subj.id);

[subjTalEvents] = getBipolarSubjElecs(subj.id, 1); % Required changing line 36 to point to correct local dir.
c_temp = {subjTalEvents.channel};
chans = vertcat(c_temp{:});

% Append lag and WAS information. Currently in
% nl_scripts/trunk/ECOG_Analyses/Helpers. Not necessary for SME
% analysis
events = append_lags(events,params.wasloc);

if params.encoding == 1
    % Filter to only the encoding items
    events = filterStruct(events,'strcmp(type,''WORD'')');
else
    % Append rec information.
    events = append_rectime(events);
    events = append_output_REC(events);
    
    % Filter to just recalled words.
    events = filterStruct(events,'strcmp(type,''REC_WORD'')');
end

% Get the appropriate power/zpower functions
power_handle = str2func(params.electype);
zscore_handle = str2func(params.zscoretype);

% Loop through all channels
for c = 1:length(chans)
    channel = chans(c,:);
    
    % Get the power
    pattern = power_handle(subj,channel,params,events); % Calculate or load the power
    
    % Z-score the power
    zpow = zscore_handle(pattern,events,params,subj,channel);
    
    % Get the frequencies
    freqs = params.freqs;
    
    % Reduce power to relevant frequencies and time bins if tstat. Does
    % not reduce if tstat = 0, does TF
    if params.tstat == 1
        % Initialize high gamma variable:
        high_gamma = nan(length(events),length(params.tbins));
        
        tempPow = zeros(size(zpow,1),length(params.tbins),length(params.fbins));
        for t = 1:length(params.tbins)
            tbin = params.tbins{t};
            for f = 1:length(params.fbins)
                fbin = freqs >= params.fbins{f}(1) & freqs <= params.fbins{f}(2);
                tempPow(:,t,f) = double(squeeze(nanmean(nanmean(zpow(:,tbin,fbin),2),3)));
                
                if strcmp(params.fbinslabel{f},'Gamma2')
                    % Two special variables are created here.
                    
                    % high_gamma,
                    % which is the high gamma power across ALL encoding
                    % events, specific to the X tbins of interest
                    high_gamma(:,t) = squeeze(nanmean(nanmean(zpow(:,tbin,fbin),3),2));
                    
                    %                     % high_gamma_all_time,
                    %                     % which is the high gamma power across ALL encoding events
                    %                     % and ALL time intervals. Obviously this is being
                    %                     % overwritten with each time loop, but I prefer that to
                    %                     % making a separate loop for freq. I want fbin to be
                    %                     % flexible so that if we add more and more frequency bins,
                    %                     % it won't just default to doing the last bin.
                    %                     high_gamma_all_time = squeeze(nanmean(zpow(:,1:2:end,fbin),3)); %#ok
                end
            end
        end
        
        zpow = tempPow;
    else
        f = find(strcmp(params.fbinslabel,'Gamma2'));
        
        fbin = freqs >= params.fbins{f}(1) & freqs <= params.fbins{f}(2);
        high_gamma = squeeze(nanmean(zpow(:,1:2:end,fbin),3)); %#ok
        
    end
    
    % For each analysis specified above, run the actual statistical tests
    for an = 1:length(params.analyses)
        
        % Get the directory name. It differentiates between tstat and TF
        % analyses
        if params.tstat
            statdir = [params.studydir,'stat/',params.analyses(an).name '_stat'];
        else
            statdir = [params.studydir,'TF/',params.analyses(an).name];
        end
        
        % Get the file name, based on electrode # and subject ID
        statfile = ['stat_bipolar_',mat2str(channel(1)),'_',mat2str(channel(2)),'_',subj.id '.mat'];
        
        % If the boot file already exists and we don't want to overwrite
        % it, the code will skip everything below and go on to the next
        % electrode
        
        if exist(fullfile(statdir,statfile),'file') && params.overwrite == 0
        else
            fprintf('%s ',mat2str(channel(1)));
            fprintf('%s\n',mat2str(channel(2)));
            % Make the statdir if it isnt there
            if ~exist(statdir,'dir');
                mkdir(statdir);
            end
            
            % Filter to the conditions of interest
            if isfield(params.analyses(an),'cond1')
                cond1_ind = inStruct(events,params.analyses(an).cond1);
                cond1 = zpow(cond1_ind,:,:);
            else
                cond1 = nan(2,length(params.tbins),length(params.fbins));
            end
            
            if isfield(params.analyses(an),'cond2')
                cond2_ind = inStruct(events,params.analyses(an).cond2);
                cond2 = zpow(cond2_ind,:,:);
            else
                cond2 = nan(2,length(params.tbins),length(params.fbins));
            end
            
            if isfield(params.analyses(an),'cond3')
                cond3_ind = inStruct(events,params.analyses(an).cond3);
                cond3 = zpow(cond3_ind,:,:);
                C3 = length(find(cond3_ind == 1));
            else
                cond3 = nan(2,length(params.tbins),length(params.fbins));
                cond12 = nan(2,length(params.tbins),length(params.fbins));
                C3 = 1;
            end
            
            C1 = length(find(cond1_ind == 1));
            C2 = length(find(cond2_ind == 1));
            
            if isfield(params.analyses(an),'cond1') & isfield(params.analyses(an),'cond2')
                if C1 < params.min_ev || C2 < params.min_ev
                    % If any condition has < min events needed events, skips for this
                    % subject/electrode
                    fprintf('A condition in the analyses %s has fewer than the required number of events. Skipping\n',params.analyses(an).name)
                    continue
                end
                if isfield(params.analyses(an),'cond3')
                    if C3 < params.min_ev
                        fprintf('A condition in the analyses %s has fewer than the required number of events. Skipping\n',params.analyses(an).name)
                        continue
                    end
                end
                
                % Run an unpaired t-test comparing the two condition powers (e.g.
                % recalled and not recalled)
                if length(fieldnames(params.analyses))-1 == 2
                    % If there are two conditions in the analysis
                    % structure, it will only compare those two
                    [~,real_ps,~,STATS] = ttest2(cond1,cond2);
                    real_ts = STATS.tstat;
                    
                elseif length(fieldnames(params.analyses))-1 == 3
                    % If there are three conditions, it will compare 1 and
                    % 2 and the average of 1/2 to 3.
                    [~,PVAL,~,STATS] = ttest2(cond1,cond2);
                    real_ts{1} = STATS.tstat;
                    real_ps{1} = PVAL;
                    
                    cond12 = ([cond1;cond2]);
                    
                    [~,PVAL,~,STATS] = ttest2(cond12,cond3);
                    real_ts{2} = STATS.tstat;
                    real_ps{2} = PVAL;
                end
                
                % Get the z-scored power difference between the two conditions.
                % Will ultimately be used in time frequency plots
                dzpow = squeeze(nanmean(cond1) - nanmean(cond2));  %#ok
            else
                dzpow = nan; real_ts = nan; %#ok
            end
            
            c1 = squeeze(nanmean(cond1)); %#ok
            c12 = squeeze(nanmean(cond12));%#ok
            c2 = squeeze(nanmean(cond2));%#ok
            c3 = squeeze(nanmean(cond3));  %#ok
            
            % Run the same unpaired t-test on bootstrapped data, performed by
            % tboot.m
            back_params = params;%#ok
            
            if params.shuffle
                [~,~,cond1_rand_avg,cond2_rand_avg,boot_ts,boot_ps] = tboot(cond1,cond2,params.iterations);%#ok
                
                save(fullfile(statdir,statfile),'cond1','cond2',...
                    'cond1_rand_avg','cond2_rand_avg','real_ts','real_ps',...
                    'dzpow','boot_ts','boot_ps','c1','c2','c3','c12','back_params',...
                    'high_gamma');
            else
                boot_ts = nan;%#ok
                boot_ps = nan; %#ok
                save(fullfile(statdir,statfile),'cond1','cond2',...
                    'real_ts','real_ps','dzpow','boot_ts','boot_ps',...
                    'c1','c2','c3','c12','back_params',...
                    'high_gamma');
            end
        end
    end
end

function subj = run_roi(subj,params)
% Loop analyses
for an = 1:length(params.analyses)
    fprintf('%s\n',params.analyses(an).name);
    % For all of the ROIs defined in run_TF_Universal_Bipolar...
    for r = 1:length(params.roi)
        fprintf('%s: ',params.roi{r});
        % Find where the TF files should be saved and set dimensions (how
        % many time intervals, how many frequencies) - differs whether you
        % are doing TStat vs. TF analysis.
        if params.tstat
            stat_dir = ([params.studydir 'stat/' params.analyses(an).name '_stat']);
            TIME = length(params.tbins);
            F = length(params.fbins);
        else
            stat_dir = ([params.studydir 'TF/' params.analyses(an).name ]);
            TIME = params.time;
            F = length(params.freqs);
        end
        
        savefile = (['Stat_bipolar_' params.roi{r} '_' subj.id '.mat']);
        
        % if the file already exists and we don't want to overwrite (==0),
        if exist(fullfile(stat_dir,savefile),'file') && params.overwrite == 0
            fprintf('%s\n','Stat file calculated skipping');
        else
            % get montage subjects if we are analyzing pyFR
            mont_subj = load('~/Experiments/pyFR/pyFR_montage_changes');
            for m = 1:length(mont_subj.subj)
                msubj(m) = strcmp(mont_subj.subj(m).original{1},subj.id);
            end
            % Get the index of channels that you want for the given
            % ROI, using bipolar_ROI
            [roi_chans] = bipolar_ROI(subj.id,params.roi{r});
            % Get all the electrodes for this subject
            [subjTalEvents] = getBipolarSubjElecs(subj.id, 1);
            c_temp = {subjTalEvents.channel};
            all_chans = vertcat(c_temp{:});
            % Get just the electrodes that are this ROI using the
            % index and full list
            chans = all_chans(logical(roi_chans{1}),:);
            
            % For subjects who aren't montage changes:
            if ~isempty(chans) && length(find(msubj == 1))==0
                % Find the number of bipolar pairs.
                [tot_chans,~] = size(chans);
                
                % Initialize variables
                cond1 = (nan(tot_chans,TIME,F));
                cond12 = (nan(tot_chans,TIME,F));
                cond2 =  (nan(tot_chans,TIME,F));
                cond3 =  (nan(tot_chans,TIME,F));
                
                t = (nan(tot_chans,length(fieldnames(params.analyses))-1,TIME,F));
                boot_ts = (nan(tot_chans,params.iterations,TIME,F));
                
                % Loop through all channels in this ROI
                for c = 1:tot_chans
                    % Load the stat_bipolar_Elec1_Elec2_SUBJ file (if it exists). Contains the
                    % delta z-score power value for the analysis of interest,
                    % as well as the real and bootstrapped t-statistics
                    
                    fprintf('%s, ',mat2str(chans(c,1)));fprintf('%s\n',mat2str(chans(c,2)));
                    statfile = ['stat_bipolar_',mat2str(chans(c,1)),'_',mat2str(chans(c,2)),'_',subj.id '.mat'];
                    
                    if exist([stat_dir '/' statfile],'file');
                        data = load ([stat_dir '/' statfile]);
                        cond1(c,:,:) = squeeze(data.c1);
                        cond2(c,:,:) = squeeze(data.c2);
                        hg(c,:,:) = squeeze(data.high_gamma);%#ok: can't initialize b/c variable events acr. subj.
                        
                        if length(fieldnames(params.analyses))-1 > 2
                            t(c,1,:,:) = squeeze(data.real_ts{1});
                            t(c,2,:,:) = squeeze(data.real_ts{2});
                            cond3(c,:,:) = squeeze(data.c3);
                            cond12(c,:,:) = squeeze(data.c12);
                        else
                            t(c,1,:,:) = squeeze(data.real_ts);
                        end
                        if params.shuffle
                            boot_ts(c,:,:,:) = data.boot_ts;
                        end
                    end
                end
                
                % Average the data over all of the chans in this ROI
                [act_chans,~,~] = size(cond1);
                if act_chans > 1
                    T = squeeze(nanmean(t));
                    C1 = squeeze(nanmean(cond1));
                    C2 = squeeze(nanmean(cond2));
                    C3 = squeeze(nanmean(cond3));
                    C12 = squeeze(nanmean(cond12));
                    T_BOOT = squeeze(nanmean(boot_ts));
                    HIGHGAMMA = squeeze(nanmean(hg));
                else
                    % don't average if there's only one electrode.
                    T = squeeze(t);
                    C1 = squeeze(cond1);
                    C2 = squeeze(cond2);
                    C3 = squeeze(cond3);
                    C12 = squeeze(cond12);
                    T_BOOT = squeeze(boot_ts);
                    HIGHGAMMA = squeeze(hg);
                end
                % Save the file and clear the big variables. Could make this a function to
                % avoid the variable issue.
                back_params = params;%#ok
                
                
                save(fullfile(stat_dir,savefile),'C1','C2','C3','C12','T','T_BOOT',...
                    't','cond1','cond2','cond3','cond12','chans','back_params',...
                    'HIGHGAMMA');
                clear C1 C2 C3 C12 T T_BOOT HIGHGAMMA hg
                
            elseif length(find(msubj ==1 )~=0)
                
                % If it is a montage subject, we want to load all of their
                % channels for this ROI, even if they might be numbered
                % differently across montage changes. We want to average
                % those values together as we would for a typical subject.
                %
                % e.g. TJ018, 1_2 is in ROI X and TJ018_1 3_4 is in ROI X,
                % but 3_4 / 1_2 aren't. We want to load all ROI X contacts
                % across the montage changes and average them together.
                % See other documentation for explanation of montage
                % subjects
                HIGHGAMMA = [];
                for mm = 1:length(mont_subj.subj(msubj).montage)+1
                    if mm == 1
                        SUBID = mont_subj.subj(msubj).original{:};
                    else
                        SUBID =  mont_subj.subj(msubj).montage{mm-1};
                    end
                    
                    [roi_chans] = bipolar_ROI(SUBID,params.roi{r});
                    [subjTalEvents] = getBipolarSubjElecs(SUBID, 1);
                    c_temp = {subjTalEvents.channel};
                    all_chans = vertcat(c_temp{:});
                    
                    chans = all_chans(logical(roi_chans{1}),:);
                    
                    if ~isempty(chans)
                        
                        [tot_chans,~] = size(chans);
                        
                        cond1 = (nan(tot_chans,TIME,F));
                        cond12 =  (nan(tot_chans,TIME,F));
                        cond2 =  (nan(tot_chans,TIME,F));
                        cond3 =  (nan(tot_chans,TIME,F));                       
                        
                        t = (nan(tot_chans,length(fieldnames(params.analyses))-1 ,TIME,F));
                        boot_ts = (nan(tot_chans,params.iterations,TIME,F));
                        
                        for c = 1:tot_chans
                            statfile = ['stat_bipolar_',mat2str(chans(c,1)),'_',mat2str(chans(c,2)),'_',SUBID '.mat'];
                            
                            if exist([stat_dir '/' statfile],'file');
                                data = load ([stat_dir '/' statfile]);
                                
                                cond1(c,:,:) = squeeze(data.c1);
                                cond2(c,:,:) = squeeze(data.c2);
                                if length(fieldnames(params.analyses))-1  > 2
                                    t(c,1,:,:) = squeeze(data.real_ts{1});
                                    t(c,2,:,:) = squeeze(data.real_ts{2});
                                    cond3(c,:,:) = squeeze(data.c3);
                                    try
                                        cond12(c,:,:) = squeeze(data.c12);
                                    end
                                else
                                    t(c,1,:,:) = squeeze(data.real_ts);
                                end                                
                                
                                hg(c,:,:) = squeeze(data.high_gamma);
                                
                                if params.shuffle
                                    boot_ts(c,:,:,:) = data.boot_ts;
                                end
                            end
                        end
                        
                        act_chans = 1;
                        try
                            [act_chans,~,~] = size(cond1);
                        end
                        
                        if act_chans > 1
                            T(mm,:,:,:) = squeeze(nanmean(t));
                            C1(mm,:,:) = squeeze(nanmean(cond1));
                            C2(mm,:,:) = squeeze(nanmean(cond2));
                            C3(mm,:,:) = squeeze(nanmean(cond3));
                            C12(mm,:,:) = squeeze(nanmean(cond12));
                            T_BOOT(mm,:,:,:) = squeeze(nanmean(boot_ts));
                            HIGHGAMMA = cat(1,HIGHGAMMA,squeeze(nanmean(hg)));
                        else
                            T(mm,:,:,:) = squeeze(t);
                            C1(mm,:,:) = squeeze(cond1);
                            C2(mm,:,:) = squeeze(cond2);
                            C3(mm,:,:) = squeeze(cond3);
                            C12(mm,:,:) = squeeze(cond12);
                            T_BOOT(mm,:,:,:) = squeeze(boot_ts);
                            HIGHGAMMA = cat(1,HIGHGAMMA,squeeze(hg));
                        end
                    end
                    clear dzpower real_ts boot_ts ts boots hg
                end
                
                if ~isempty(chans)
                    T = squeeze(nanmean(T));
                    C1 = squeeze(nanmean(C1));
                    C2 = squeeze(nanmean(C2));
                    C3 = squeeze(nanmean(C3));
                    C12 = squeeze(nanmean(C12));
                    T_BOOT = squeeze(nanmean(T_BOOT));
                    
                    % Save the file and clear the big variables. Could make this a function to
                    % avoid the variable issue.
                    back_params = params;%#ok
                    
                    save(fullfile(stat_dir,savefile),'C1','C2','C3','C12','T',...
                        'T_BOOT','chans','back_params','HIGHGAMMA');
                    clear C1 C2 C3 C12 T T_BOOT HIGHGAMMA
                end
            else
                fprintf('%s\n','No electrodes in this ROI');
            end
        end
    end
end